"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAtomizerError = exports.getAtomizedRootTask = exports.makeAcyclic = exports.findCycle = void 0;
const configuration_1 = require("../config/configuration");
const nx_cloud_utils_1 = require("../utils/nx-cloud-utils");
const serialize_target_1 = require("../utils/serialize-target");
function _findCycle(graph, id, visited, path) {
    if (visited[id])
        return null;
    visited[id] = true;
    for (const d of graph.dependencies[id]) {
        if (path.includes(d))
            return [...path, d];
        const cycle = _findCycle(graph, d, visited, [...path, d]);
        if (cycle)
            return cycle;
    }
    return null;
}
function findCycle(graph) {
    const visited = {};
    for (const t of Object.keys(graph.dependencies)) {
        visited[t] = false;
    }
    for (const t of Object.keys(graph.dependencies)) {
        const cycle = _findCycle(graph, t, visited, [t]);
        if (cycle)
            return cycle;
    }
    return null;
}
exports.findCycle = findCycle;
function _makeAcyclic(graph, id, visited, path) {
    if (visited[id])
        return;
    visited[id] = true;
    const deps = graph.dependencies[id];
    for (const d of [...deps]) {
        if (path.includes(d)) {
            deps.splice(deps.indexOf(d), 1);
        }
        else {
            _makeAcyclic(graph, d, visited, [...path, d]);
        }
    }
    return null;
}
function makeAcyclic(graph) {
    const visited = {};
    for (const t of Object.keys(graph.dependencies)) {
        visited[t] = false;
    }
    for (const t of Object.keys(graph.dependencies)) {
        _makeAcyclic(graph, t, visited, [t]);
    }
    graph.roots = Object.keys(graph.dependencies).filter((t) => graph.dependencies[t].length === 0);
}
exports.makeAcyclic = makeAcyclic;
function getAtomizedRootTask(task, taskGraph, projectGraph) {
    const project = projectGraph.nodes[task.target.project]?.data;
    const target = project.targets[task.target.target];
    // default case, e2e-ci targets should fail in default task runner
    if (target.metadata?.nonAtomizedTarget !== undefined) {
        return task.target;
    }
    const targetGroups = project.metadata?.targetGroups;
    if (!targetGroups) {
        return undefined;
    }
    // checking target groups to see if the target is part of at atomized group
    for (const group in targetGroups) {
        if (targetGroups[group].includes(task.target.target)) {
            const rootTarget = targetGroups[group]?.find((t) => task.target.target.startsWith(`${t}--`));
            // we want to fail individual atomized targets only if the root target is also being run
            const rootTaskId = (0, serialize_target_1.serializeTarget)(task.target.project, rootTarget, task.target.configuration);
            if (rootTarget &&
                project.targets[rootTarget].metadata?.nonAtomizedTarget !== undefined &&
                !!taskGraph.tasks[rootTaskId]) {
                return taskGraph.tasks[rootTaskId].target;
            }
        }
    }
    return undefined;
}
exports.getAtomizedRootTask = getAtomizedRootTask;
function getAtomizerError(atomizedRootTaskId, nonAtomizedTask) {
    const usingNxCloud = (0, nx_cloud_utils_1.isNxCloudUsed)((0, configuration_1.readNxJson)());
    const outputLines = [
        `The ${atomizedRootTaskId} task should only be run with ${usingNxCloud ? 'distribution' : 'Nx Cloud'}.`,
        '',
        `Please enable ${usingNxCloud ? 'distribution with Nx Agents' : 'Nx Cloud'} or use the slower ${nonAtomizedTask} task.`,
        'Learn more at https://nx.dev/ci/features/split-e2e-tasks#nx-cloud-is-required-to-run-atomized-tasks',
    ];
    return outputLines.join('\n');
}
exports.getAtomizerError = getAtomizerError;
